﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Core;

namespace HeuristicLab.Algorithms.ParameterlessPopulationPyramid {
  public static class ListExtensions {
    public static IList<T> Swap<T>(this IList<T> list, int indexA, int indexB) {
      T tmp = list[indexA];
      list[indexA] = list[indexB];
      list[indexB] = tmp;
      return list;
    }

    public static IList<T> ShuffleInPlace<T>(this IList<T> list, IRandom random) {
      return list.ShuffleInPlace(random, 0, list.Count - 1);
    }
    public static IList<T> ShuffleInPlace<T>(this IList<T> list, IRandom random, int maxIndex) {
      return list.ShuffleInPlace(random, 0, maxIndex);
    }
    public static IList<T> ShuffleInPlace<T>(this IList<T> list, IRandom random, int minIndex, int maxIndex) {
      for (int i = maxIndex; i > minIndex; i--) {
        int swapIndex = random.Next(minIndex, i + 1);
        list.Swap(i, swapIndex);
      }
      return list;
    }

    public static IEnumerable<T> ShuffleList<T>(this IList<T> source, IRandom random) {
      for (int i = source.Count - 1; i > 0; i--) {
        // Swap element "i" with a random earlier element (including itself)
        int swapIndex = random.Next(i + 1);
        source.Swap(i, swapIndex);
        yield return source[i];
      }
      yield return source[0];
    }
  }
}
